#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "bmk.h"
#include "buffer.h"


 /*
  Bookmarks are multi-level. There are 4 kinds of bookmark entries:
  - bookmark folder  'F'
  - bookmark         'B'
  - folder end       'E'

  The general on-disk format is:

    [size] [type] [page] [title]

  If type is 'E'  then the page and title fields are missing.
  For 'F' type entries, the page field is missing. All fields are
  present for 'B' entries.

  The whole bookmark section is a single entity. There is no table etc.
  associated with it.
 ***********************************************************************/

static void bput_int4(buffer_t *buf, unsigned int K)
{
  unsigned char V[4];
  int i;
  for(i=0;i<4;i++) { V[i]= K; K>>=8; }
  buffer_write(buf, V,4);

}

static void bput_int1(buffer_t *buf, unsigned int K)
{
  unsigned char V;
  V= K;
  buffer_write(buf, &V,1);
}


static void put_end(buffer_t *buf)
{
  bput_int4(buf,1);
  bput_int1(buf,'E');
}

static void bput_str(buffer_t *buf, char *str)
{
  buffer_write(buf, str, strlen(str)+1);
}

static void put_bookmark(buffer_t *buf,bmk_t *B)
{
  
  if (B->type=='B')
  {
      // [size] [type] [page] [title]
    bput_int4(buf, 1 + 4 + strlen(B->title) + 1);  // size
    bput_int1(buf, 'B');
    bput_int4(buf, B->page);
    bput_str(buf, B->title);
    return ;
  }
  if (B->type!='F') return ;

      // [size] [type] [title]
  bput_int4(buf, 1 + strlen(B->title) + 1);  // size
  bput_int1(buf, 'F');
  bput_str(buf, B->title);

  if (B->children)
  {
    bmk_t *C;
    for(C=B->children->next;C!=B->children;C=C->next)
      put_bookmark(buf,C);
  }
  put_end(buf);
}

unsigned char *bmk_encode(bmk_t *top, unsigned int *Rsz)
{
  buffer_t *buf;
  bmk_t *B;
  unsigned char *R;
  size_t sz;
  int i;

  buf= buffer_new(200);
  bput_int4(buf, 0); // size

  if (top->children)
  {
    for(B=top->children->next;B!=top->children;B=B->next)
      put_bookmark(buf,B);
  }
  put_end(buf);

  buffer_free(buf, (void**) &R, &sz);

  *Rsz= sz;
  sz-= 4; // the size in the buffer doesn't include
          // the size field itself.
  for(i=0;i<4;i++)
   { R[i]= sz; sz>>= 8; }

  return R;
}

