
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <zlib.h>
#include "gmap.h"
#include "bitmap.h"
#include "pngio.h"

#define MAGIC_SIZE 8

static void set_fpos(FILE *f, unsigned int pos)
{
  if (fseek(f, pos, SEEK_SET))
    exit(fprintf(stderr,"can't seek in output file: %s\n",strerror(errno)));
}

static int bmap_section_offset, bmap_section_size,
           gmap_section_offset, gmap_section_size,
           page_section_offset, page_section_size;


static unsigned int *gmap_table;
static unsigned int n_gmap;
static gmap_t **gmap;


static void read_bytes(FILE *f, void *dst, int len)
{
  int R;
  R= fread(dst,1,len,f);
  if (R!=len)
    exit(fprintf(stderr,"error during read\n"));
}

static unsigned int dint4(unsigned char *V)
{
  return V[0] | (V[1]<<8) | (V[2]<<16) | (V[3]<<24);
}

static unsigned int read_int4(FILE *f)
{
  unsigned char B[4];
  read_bytes(f,B,4);
  return dint4(B); 
}

static unsigned int dint2(unsigned char *K)
{
  return K[0] | (K[1]<<8);
}

static void read_hdr(FILE *f)
{
  set_fpos(f, MAGIC_SIZE);
  bmap_section_size= read_int4(f);
  bmap_section_offset= read_int4(f);
  gmap_section_size= read_int4(f);
  gmap_section_offset= read_int4(f);
  page_section_size= read_int4(f);
  page_section_offset= read_int4(f);
}


static void read_gmap_table(FILE *f)
{
  set_fpos(f, gmap_section_offset);
  n_gmap= read_int4(f);
  gmap_table= malloc(4*n_gmap);
  read_bytes(f, gmap_table, 4*n_gmap);
  gmap= calloc(sizeof(gmap[0]),n_gmap);
}

static void dump_gmap(FILE *f,int num)
{
  int sz;
  char *b;
  FILE *out;
  static int last= 0;
  char fn[200];

  set_fpos(f, gmap_table[num]);
  sz= read_int4(f);
  b= malloc(sz);
  read_bytes(f, b, sz);

  sprintf(fn,"gma%04d.png",last++);
  out= fopen(fn,"wb");
  if (!out) exit(fprintf(stderr,"can't open %s for writing\n",fn));

  fwrite(b+4, 1, sz-4, out);
  fclose(out);
  free(b);
}





int main(int argc,char **argv)
{
  char *ifile;
  FILE *inp;
  int i;

  ifile= argv[1];
  
  inp= fopen(ifile, "rb");
  if (!inp) exit(fprintf(stderr,"can't open input\n"));
  read_hdr(inp);
  read_gmap_table(inp);

  for(i=0;i<n_gmap;i++)
    dump_gmap(inp,i);

  fclose(inp);
  return 0;
}
