#include <parseargs.h>
#include <GList.h>
#include <GlobalParams.h>
#include <PDFDoc.h>
#include <config.h>
#include <TextString.h>
#include <SplashBitmap.h>
#include <Splash.h>
#include <SplashOutputDev.h>
#include <Outline.h>
#include "bmk.h"
#include "pdf.h"

static PDFDoc             *doc;
static SplashOutputDev    *out;
static double              resolution;

int pdf_begin(char *fn, double resol)
{
  SplashColor  paperColor;

  globalParams = new GlobalParams("");
  globalParams->setupBaseFonts(NULL);
  // if (quiet) globalParams->setErrQuiet(quiet);
  if (!globalParams->setAntialias((char*) "no"))
  {
     fprintf(stderr, "Can not disable font antialiasing\n");
     goto err_doc;
  }
  if (!globalParams->setVectorAntialias((char*) "no")) 
  {
     fprintf(stderr, "Can not disable vector antialiasing\n");
     goto err_doc;
  }

  doc= new PDFDoc(fn, NULL, NULL);
  if (!doc->isOk()) goto err_doc;

  paperColor[0]= 0xff;
  out= new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);
  out->startDoc(doc->getXRef());

  resolution= resol;

  return 0;

err_doc:
  pdf_end();
  return 1;
}

int pdf_page(int pn, int *Rw, int *Rh, unsigned char **Rdata)
{
  SplashBitmap *b;
  doc->displayPage(out, pn, resolution, resolution, 0, gFalse, gTrue, gFalse);
  b= out->getBitmap();
  *Rw= b->getWidth();
  *Rh= b->getHeight();
  *Rdata= b->getDataPtr();
  return 0;
}

int pdf_npages()
{
  return doc->getNumPages();
}

void pdf_end()
{
  if (out) { delete out; out= NULL; }
  if (doc) { delete doc; doc= NULL; }
  if (globalParams) { delete globalParams; globalParams= NULL; }
}

static char *get_title(OutlineItem *item)
{
  TextString *T;                                                                
  char *s;                                                                   
  GString *S;                                                                   
  T= item->getTitleTextString(); if (!T) goto empty;
  S= T->toUTF8();                if (!S) goto empty;
  s= S->getCString();            if (!s) goto empty;
  s= strdup(s);                  if (!s) goto empty;
  return s;
empty:
  return strdup("");
}

static bmk_t *bmk_node(int type)
{
  bmk_t *B;
  B= (bmk_t*) calloc(1,sizeof(*B));
  B->type= type;
  B->prev= B->next= B;

  if (B->type=='F')
  {
    B->children= bmk_node(0);
    B->children->parent= B;
  }
  return B;
}

static void bmk_add_child(bmk_t *P, bmk_t *B)
{
  B->prev= P->children->prev;
  B->next= P->children;
  P->children->prev->next= B;
  P->children->prev= B;
  B->parent= P;
}

static void walk_bmk(PDFDoc *doc, OutlineItem *item, bmk_t *parent,int level);

static void walk_list(PDFDoc *doc, GList *list, bmk_t *parent,int level)
{
  int i,n;
  n= list->getLength();
  for(i=0;i<n;i++)
    walk_bmk(doc, (OutlineItem*) list->get(i), parent, level);
}

static void walk_bmk(PDFDoc *doc, OutlineItem *item, bmk_t *parent,int level)
{
  bmk_t *B;
  GList *list;

  item->open();
  if (item->hasKids()) 
  {
    B= bmk_node('F'); 
    walk_list(doc, item->getKids(), B, level+1);
  }
  else
  {
    B= bmk_node('B');
  }
  B->level= level;
  B->parent= parent;
  B->title= get_title(item);
  B->page= doc->getOutlineTargetPage(item);
  bmk_add_child(parent, B);
  if (B->type=='F') list= item->getKids();
}

bmk_t *pdf_bookmarks()
{
  bmk_t *top;
  Outline *outline;
  outline=doc->getOutline();
  if (!outline) return NULL;
  top= bmk_node('F');
  walk_list(doc, outline->getItems(), top, 1);
  return top;
}

